/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2011, 2012 Centre Tecnologic de Telecomunicacions de Catalunya (CTTC)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Tao WANG <tao.wang@inria.fr>
 *
 */

#include "ns3/core-module.h"
#include "ns3/mobility-module.h"
#include "ns3/wiplan.h"

using namespace ns3;

class WiplanExample {
public:
	WiplanExample();
	~WiplanExample();
private:
	Ptr<PropagationLossModel> m_loss;
	Ptr<MobilityModel> m_firstMobility;
	Ptr<MobilityModel> m_secondMobility;
	Time m_step;
	EventId m_nextEvent;
	void Next();
};

WiplanExample::WiplanExample() :
		m_step(Seconds(0.0002)) //1/5000 part of the second
{
	m_loss = CreateObject<wiplan>();
	m_firstMobility = CreateObject<ConstantPositionMobilityModel>();
	m_secondMobility = CreateObject<ConstantPositionMobilityModel>();
	m_firstMobility->SetPosition(Vector(13, 25, 0));
	m_secondMobility->SetPosition(Vector(100, 30, 0));
	Next();
//	m_nextEvent = Simulator::Schedule(m_step, &MRFDPFPropagationExample::Next,
//			this);
}

WiplanExample::~WiplanExample() {

}

void WiplanExample::Next() {
//	m_nextEvent = Simulator::Schedule(m_step, &MRFDPFPropagationExample::Next,
//			this);
//	std::cout << Simulator::Now().GetMilliSeconds() << " "
        std::cout << "The postition of A is : " << m_firstMobility->GetPosition() << std::endl;
        std::cout << "The postition of B is : " << m_secondMobility->GetPosition() << std::endl;
	std::cout << "The path loss from A to B is : "	<< m_loss->CalcRxPower(0, m_firstMobility, m_secondMobility)
			<< std::endl;
        
}

int main(int argc, char *argv[]) {
//	Config::SetDefault("ns3::MRFDPFProcess::NumberOfOscillators",
//			UintegerValue(100));
//	CommandLine cmd;
//	cmd.Parse(argc, argv);
	WiplanExample example;
	Simulator::Stop(Seconds(1000));
	Simulator::Run();
	Simulator::Destroy();
	return 0;
}


