/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2011, 2012
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Tao WANG <tao.wang@inria.fr>
 *
 */

const int E = 0;

const int W = 1;

const int S = 2;

const int N = 3;

const int HORIZ = 0;

const int VERTI = 1;

const int OUT = 0;

const int IN = 1;

const double PI = 3.141592654;

const double C0 = 3.0E8;

const double DR = 0.1;

const double f0_ISM = 2.4E9;

const double f0_DFS = 5.0E9;

const double f0_test = 4.8E8;

#include <math.h>
#include <fstream>
#include <ns3/log.h>
#include <ns3/double.h>
#include <ns3/enum.h>
#include <ns3/mobility-model.h>
#include "wiplan.h"

#include "gsl/gsl_complex.h"
#include "gsl/gsl_complex_math.h"
#include "gsl/gsl_linalg.h"
#include "gsl/gsl_matrix.h"
#include "gsl/gsl_blas.h"

int cgemm(CBLAS_TRANSPOSE TransA, CBLAS_TRANSPOSE TransB,
		const gsl_complex alpha, const gsl_matrix_complex* A,
		const gsl_matrix_complex* B, const gsl_complex beta,
		gsl_matrix_complex* C) {
	const size_t M = C->size1;
	const size_t N = C->size2;
	const size_t MA = (TransA == CblasNoTrans) ? A->size1 : A->size2;
	const size_t NA = (TransA == CblasNoTrans) ? A->size2 : A->size1;
	const size_t MB = (TransB == CblasNoTrans) ? B->size1 : B->size2;
	const size_t NB = (TransB == CblasNoTrans) ? B->size2 : B->size1;

	if (M == MA && N == NB && NA == MB) /* [MxN] = [MAxNA][MBxNB] */
	{
		cblas_zgemm(CblasRowMajor, TransA, TransB, int(M), int(N), int(NA),
				GSL_COMPLEX_P (&alpha), A->data, int(A->tda), B->data,
				int(B->tda), GSL_COMPLEX_P (&beta), C->data, int(C->tda));
		return 0;
	} else {
		GSL_ERROR("invalid length", GSL_EBADLEN);
		return -1;
	}
}

int cgemv(CBLAS_TRANSPOSE TransA, const gsl_complex alpha,
		const gsl_matrix_complex * A, const gsl_vector_complex * X,
		const gsl_complex beta, gsl_vector_complex * Y) {
	const size_t M = A->size1;
	const size_t N = A->size2;

	if ((TransA == CblasNoTrans && N == X->size && M == Y->size)
			|| (TransA == CblasTrans && M == X->size && N == Y->size)
			|| (TransA == CblasConjTrans && M == X->size && N == Y->size)) {
		cblas_zgemv(CblasRowMajor, TransA, int(M), int(N),
				GSL_COMPLEX_P (&alpha), A->data, int(A->tda), X->data,
				int(X->stride), GSL_COMPLEX_P (&beta), Y->data, int(Y->stride));
		return GSL_SUCCESS;
	} else {
		GSL_ERROR("invalid length", GSL_EBADLEN);
		return -1;
	}
}

class BlocMatrix {

public:
	gsl_matrix_complex *propagDown_ToA[4];
	gsl_matrix_complex *propagDown_ToB[4];
	gsl_matrix_complex *propagSource_AtoA;
	gsl_matrix_complex *propagSource_AtoB;
	gsl_matrix_complex *propagSource_BtoA;
	gsl_matrix_complex *propagSource_BtoB;
	gsl_matrix_complex *W[4][4];
	BlocMatrix();
	virtual ~BlocMatrix();
};
BlocMatrix::BlocMatrix() {
	for (int i = 0; i < 4; i++) {
		propagDown_ToA[i] = NULL;
		propagDown_ToB[i] = NULL;
		for (int j = 0; j < 4; j++)
			W[i][j] = NULL;
	}
	propagSource_AtoA = NULL;
	propagSource_AtoB = NULL;
	propagSource_BtoA = NULL;
	propagSource_BtoB = NULL;
}
BlocMatrix::~BlocMatrix() {
}

class Material {
	double indice;
	double n;
	double alpha;
public:
	Material(double indice, double n, double alpha);
	virtual ~Material();
};
Material::Material(double indice, double n, double alpha) {
	this->indice = indice;
	this->n = n;
	this->alpha = alpha;

}
Material::~Material() {
	// TODO Auto-generated destructor stub
}

class Source {
public:
	int row, column;
	size_t size1, size2;
	/*Phase du signal*/
	double phase;
	/*Amplitude du signal*/
	double amplitude;
	Source();
	Source(int raw, int column, size_t size1, size_t size2);
	virtual ~Source();
};
Source::Source(int row, int column, size_t size1, size_t size2) {
	this->row = row;
	this->column = column;
	this->size1 = size1;
	this->size2 = size2;

}

Source::Source() {
}

Source::~Source() {
	// TODO Auto-generated destructor stub
}

class EnvironmentFile {
	int materialNum; //number of materials
	size_t size1, size2; //size of the environment matrix
	std::vector<float> mat;
	std::vector<std::vector<double> > matTable;
	gsl_matrix *environmentMatrix; //enviroment matrix
public:
	EnvironmentFile(const char* coeffFile);
	virtual ~EnvironmentFile();

	void readCoeffFile(const char* coeffFile);

	std::vector<std::vector<double> > getMatTable() const {
		return matTable;
	}

	int getMaterialNum() const {
		return materialNum;
	}

	size_t getSize1() const {
		return size1;
	}

	size_t getSize2() const {
		return size2;
	}
	gsl_matrix* getEnvironmentMatrix() const {
		return environmentMatrix;
	}
};
EnvironmentFile::EnvironmentFile(const char* coeffFile) {
	readCoeffFile(coeffFile);

}
EnvironmentFile::~EnvironmentFile() {
	// TODO Auto-generated destructor stub
}
void EnvironmentFile::readCoeffFile(const char* coeffFile) {
	std::ifstream infile;
	infile.open(coeffFile);
	infile >> materialNum;
//	std::cout << "Number of material: " << materialNum << std::endl;

	for (int i = 0; i < materialNum; i++) {
		std::vector<double> tmp;
		double n = 0, m = 0, r = 0;
		infile >> n >> m >> r;
		tmp.push_back(n);
		tmp.push_back(m);
		tmp.push_back(r);
		matTable.push_back(tmp);
		tmp.clear();
	}

	infile >> size2;
	infile >> size1;
//	std::cout << "size1: " << size1 << std::endl;
//	std::cout << "size2: " << size2 << std::endl;
	environmentMatrix = gsl_matrix_alloc(size1, size2);
	for (size_t i = 0; i < size1; i++) {
		for (size_t j = 0; j < size2; j++) {
			int n = 0;
			infile >> n;
			gsl_matrix_set(environmentMatrix, i, j, n);
		}
	}
	infile.close();
}

class BlocType {
public:
	bool bounds[4];
	BlocType *childR, *childL;
	BlocMatrix bm;
	bool matrix;
	bool downMatrix;
	int splitDir;
	size_t size1, size2;
	int indice;
	BlocType(size_t size1, size_t size2, int indice);
	virtual ~BlocType();

	bool isUnitary() {
		if (this->size1 == 1 && this->size2 == 1)
			return true;
		else
			return false;
	}
	BlocMatrix calculBlocMatrix(gsl_matrix_complex *btmA[4][4],
			gsl_matrix_complex *btmB[4][4]);
	gsl_matrix_complex *gsl_matrix_complex_invert(gsl_matrix_complex *m);
	gsl_matrix_complex *gsl_matrix_complex_concat(gsl_matrix_complex *A,
			gsl_matrix_complex *B, bool dir);
	gsl_matrix_complex *cgemm_concat(gsl_matrix_complex *A,
			gsl_matrix_complex *B1, gsl_matrix_complex *B2,
			gsl_matrix_complex *C1, gsl_matrix_complex *C2);
	void setPropagDownMatrix(BlocType *childl, BlocType *childr);
	void printMatrix(gsl_matrix_complex *m[4][4]);
};
BlocType::BlocType(size_t size1, size_t size2, int indice) {
	this->size1 = size1;
	this->size2 = size2;
	this->indice = indice;
	this->childL = NULL;
	this->childR = NULL;
	this->matrix = false;
	this->downMatrix = false;
	bounds[0] = false;
	bounds[1] = false;
	bounds[2] = false;
	bounds[3] = false;
}
BlocType::~BlocType() {
	// TODO Auto-generated destructor stub
}
BlocMatrix BlocType::calculBlocMatrix(gsl_matrix_complex *btmA[4][4],
		gsl_matrix_complex *btmB[4][4]) {
	//printMatrix(btmA);
	//printMatrix(btmB);
//	std::cout << std::endl;
	BlocMatrix bm;
	int e, w, s, n;
	gsl_matrix_complex *MM[2][8];
	gsl_matrix_complex *propagW[4][4];
	for (int i = 0; i < 4; i++)
		for (int j = 0; j < 4; j++)
			propagW[i][j] = NULL;

	if (splitDir == HORIZ) {
		e = E;
		w = W;
		s = S;
		n = N;
	} else {
		e = S;
		w = N;
		s = E;
		n = W;
	}

	gsl_matrix_complex *pweB, *pewA, *pww, *pee;
	pweB = gsl_matrix_complex_alloc(btmB[w][e]->size1, btmB[w][e]->size2);
	pewA = gsl_matrix_complex_alloc(btmA[e][w]->size1, btmA[e][w]->size2);
	gsl_matrix_complex_memcpy(pweB, btmB[w][e]);
	gsl_matrix_complex_memcpy(pewA, btmA[e][w]);
//	gsl_matrix_complex_fprintf(stdout, pweB, "%f");
//	gsl_matrix_complex_fprintf(stdout, pewA, "%f");
//	weB = gsl_matrix_complex_alloc(btmB[w][e].size1, btmB[w][e].size2);
//	gsl_matrix_complex_memcpy(weB, &btmB[w][e]);
//	ewA = gsl_matrix_complex_alloc(btmB[e][w].size1, btmB[e][w].size2);
//	gsl_matrix_complex_memcpy(ewA, &btmB[e][w]);

	pww = gsl_matrix_complex_alloc(pewA->size1, pewA->size1);
	gsl_matrix_complex_set_identity(pww);
	//clock_t start = clock();
	cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(-1, 0), pweB, pewA,
			gsl_complex_rect(1, 0), pww);
	//clock_t finish = clock();

//	if (pww->size1 > 100 && pww->size2 > 100) {
//		std::cout << pww->size1 << " " << pww->size2 << " "
//				<< (finish - start) / 1000 << std::endl;
//	}

//gsl_matrix_complex_fprintf(stdout, pww, "%f");
//	clock_t start = clock();
	pww = gsl_matrix_complex_invert(pww);
//	clock_t finish = clock();
//
//	if (pww->size1 > 100 && pww->size2 > 100) {
//		std::cout << pww->size1 << " " << pww->size2 << " "
//				<< (finish - start) / 1000 << std::endl;
//	}

//gsl_matrix_complex_fprintf(stdout, pww, "%f");
	pee = gsl_matrix_complex_alloc(pww->size1, pww->size2);
	gsl_matrix_complex_memcpy(pee, pww);
	gsl_matrix_complex_transpose(pee);
	MM[0][6] = gsl_matrix_complex_alloc(pww->size1, pww->size2);
	gsl_matrix_complex_memcpy(MM[0][6], pww);
	MM[0][7] = gsl_matrix_complex_alloc(pweB->size1, pee->size2);
	cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0), pweB, pee,
			gsl_complex_rect(0, 0), MM[0][7]);
	MM[1][6] = gsl_matrix_complex_alloc(pewA->size1, pww->size2);
	cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0), pewA, pww,
			gsl_complex_rect(0, 0), MM[1][6]);
	MM[1][7] = gsl_matrix_complex_alloc(pee->size1, pee->size2);
	gsl_matrix_complex_memcpy(MM[1][7], pee);

	for (int sortie = 0; sortie < 2; sortie++) {
		if (bounds[w]) { // si il y a un voisin a l'west
			MM[sortie][0] = gsl_matrix_complex_alloc(MM[sortie][7]->size1,
					btmA[e][e]->size2);
			cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0),
					MM[sortie][7], btmA[e][e], gsl_complex_rect(0, 0),
					MM[sortie][0]);
			//MM[sortie][0] = MM[sortie][7].zMult(ScatMatA[e][e], null);
		}
		if (bounds[e]) { // si il y a un voisin a l'est
			MM[sortie][1] = gsl_matrix_complex_alloc(MM[sortie][6]->size1,
					btmB[w][w]->size2);
			cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0),
					MM[sortie][6], btmB[w][w], gsl_complex_rect(0, 0),
					MM[sortie][1]);
			//MM[sortie][1] = MM[sortie][6].zMult(ScatMatB[w][w], null);
		}
		if (bounds[n]) { // si il y a un voisin au nord
			MM[sortie][2] = gsl_matrix_complex_alloc(MM[sortie][7]->size1,
					btmA[e][s]->size2);
			cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0),
					MM[sortie][7], btmA[e][s], gsl_complex_rect(0, 0),
					MM[sortie][2]);
			//MM[sortie][2] = MM[sortie][7].zMult(ScatMatA[e][s], null);
			MM[sortie][3] = gsl_matrix_complex_alloc(MM[sortie][6]->size1,
					btmB[w][s]->size2);
			cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0),
					MM[sortie][6], btmB[w][s], gsl_complex_rect(0, 0),
					MM[sortie][3]);
			//MM[sortie][3] = MM[sortie][6].zMult(ScatMatB[w][s], null);
		}
		if (bounds[s]) { // si il y a un bloc au sud
			MM[sortie][4] = gsl_matrix_complex_alloc(MM[sortie][7]->size1,
					btmA[e][n]->size2);
			cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0),
					MM[sortie][7], btmA[e][n], gsl_complex_rect(0, 0),
					MM[sortie][4]);
			//MM[sortie][4] = MM[sortie][7].zMult(ScatMatA[e][n], null);
			MM[sortie][5] = gsl_matrix_complex_alloc(MM[sortie][6]->size1,
					btmB[w][n]->size2);
			cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0),
					MM[sortie][6], btmB[w][n], gsl_complex_rect(0, 0),
					MM[sortie][5]);
			//MM[sortie][5] = MM[sortie][6].zMult(Scatgsl_matrix_complex_fprintf(stdout, pweB, "%f");MatB[w][n], null);
		}
	}

	if (bounds[e]) { // si voisin Est peut exister
		// West->Est, retour de flux

		propagW[e][w] = gsl_matrix_complex_alloc(btmB[e][w]->size1,
				btmB[e][w]->size2);
		gsl_matrix_complex_memcpy(propagW[e][w], btmB[e][w]);
		cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0), btmB[e][e],
				MM[1][1], gsl_complex_rect(1, 0), propagW[e][w]);
		//ScatMat[e][w] = ScatMatB[e][e].zMultNew(MM[1][1], ScatMatB[e][w], 1, 1,
		//false, false);
		// Est->Est
		if (bounds[w]) { // si il y a un voisin a l'ouest
			propagW[e][e] = gsl_matrix_complex_alloc(btmB[e][e]->size1,
					MM[1][0]->size2);
			cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0),
					btmB[e][e], MM[1][0], gsl_complex_rect(0, 0),
					propagW[e][e]);
			//cout << "ee" << ScatMat[e][e] << endl;
			//ScatMat[e][e] = ScatMatB[e][e].zMult(MM[1][0], null);
			propagW[w][w] = gsl_matrix_complex_alloc(propagW[e][e]->size2,
					propagW[e][e]->size1);
			gsl_matrix_complex_transpose_memcpy(propagW[w][w], propagW[e][e]);
			//cout << "ww" << ScatMat[w][w] << endl;
			//ScatMat[w][w] = ScatMat[e][e].transpose();

		}

		if (bounds[n]) { // si il y a un voisin au nord
			propagW[e][s] = gsl_matrix_complex_alloc(btmB[e][e]->size1,
					MM[1][2]->size2 + MM[1][3]->size2);
			propagW[e][s] = cgemm_concat(btmB[e][e], MM[1][2], MM[1][3], NULL,
					btmB[e][s]);
//			propagW[e][s] = ConcatGmm(btmB[e][e], MM[1][2], MM[1][3],
//					btmB[e][s], true);
			//cout << "es" << ScatMat[e][s] << endl;
			//ScatMat[e][s] = ScatMatB[e][e].zMultConcatNew(MM[1][2], MM[1][3],
			//null, ScatMatB[e][s], 1, 1, false, false);
			propagW[n][w] = gsl_matrix_complex_alloc(propagW[e][s]->size2,
					propagW[e][s]->size1);
			gsl_matrix_complex_transpose_memcpy(propagW[n][w], propagW[e][s]);
			//cout << "nw" << ScatMat[n][w] << endl;
		}
		// Nord->Est
		if (bounds[s]) { // si il y a un voisin au sud
			propagW[e][n] = cgemm_concat(btmB[e][e], MM[1][4], MM[1][5], NULL,
					btmB[e][n]);
//			propagW[e][n] = ConcatGmm(btmB[e][e], MM[1][4], MM[1][5],
//					btmB[e][n], true);
			//cout << "en" << ScatMat[e][n] << endl;
			//ublas::blas_3::gmm(ScatMat[e][s], 1, 1, ScatMatB[e][e], ConcatMatrix(MM[1][4], MM[1][5]));
			//ScatMat[e][n] = ScatMatB[e][e].zMultConcatNew(MM[1][4], MM[1][5],
			//null, ScatMatB[e][n], 1, 1, false, false);
			propagW[s][w] = gsl_matrix_complex_alloc(propagW[e][n]->size2,
					propagW[e][n]->size1);
			gsl_matrix_complex_transpose_memcpy(propagW[s][w], propagW[e][n]);
			//cout << "sw" << ScatMat[s][w] << endl;
		}
	}

	// propag vers l'ouest
	if (bounds[w]) { // si voisin West peut exister
		// Est->West, retour de flux
		propagW[w][e] = gsl_matrix_complex_alloc(btmB[w][e]->size1,
				btmB[w][e]->size2);
		gsl_matrix_complex_memcpy(propagW[w][e], btmA[w][e]);
		cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0), btmA[w][w],
				MM[0][0], gsl_complex_rect(1, 0), propagW[w][e]);
		//ScatMat[w][e] = ScatMatA[w][w].zMultNew(MM[0][0], ScatMatA[w][e], 1, 1,
		//false, false);
		// Sud->West
		if (bounds[n]) { // si il y a un voisin au nord
			propagW[w][s] = cgemm_concat(btmA[w][w], MM[0][2], MM[0][3],
					btmA[w][s], NULL);
//			propagW[w][s] = ConcatGmm(btmA[w][w], MM[0][2], MM[0][3],
//					btmA[w][s], false);
			//ScatMat[w][s] = ScatMatA[w][w].zMultConcatNew(MM[0][2], MM[0][3],
			//ScatMatA[w][s], null, 1, 1, false, false);
			/*
			 * BlocMatrix.MultNewConcat( ScatMatA[w][w], MM[0][2], MM[0][3],
			 * ScatMatA[w][s], null, 1, 1, false, false);
			 */
			propagW[n][e] = gsl_matrix_complex_alloc(propagW[w][s]->size2,
					propagW[w][s]->size1);
			gsl_matrix_complex_transpose_memcpy(propagW[n][e], propagW[w][s]);
			//ScatMat[n][e] = ScatMat[w][s].transpose();
		}
		// Nord->West
		if (bounds[s]) { // si il y a un voisin au sud
			propagW[w][n] = cgemm_concat(btmA[w][w], MM[0][4], MM[0][5],
					btmA[w][n], NULL);
//			propagW[w][n] = ConcatGmm(btmA[w][w], MM[0][4], MM[0][5],
//					btmA[w][n], false);
			//ScatMat[w][n] = ScatMatA[w][w].zMultConcatNew(MM[0][4], MM[0][5],
			//ScatMatA[w][n], null, 1, 1, false, false);
			/*
			 * BlocMatrix.MultNewConcat( ScatMatA[w][w], MM[0][4], MM[0][5],
			 * ScatMatA[w][n], null, 1, 1, false, false);
			 */
			propagW[s][e] = gsl_matrix_complex_alloc(propagW[w][n]->size2,
					propagW[w][n]->size1);
			gsl_matrix_complex_transpose_memcpy(propagW[s][e], propagW[w][n]);
			//ScatMat[s][e] = ScatMat[w][n].transpose();
		}
	}
	// propag vers le sud
	if (bounds[s]) { // si voisin Sud peut exister
		// Est->Sud : deja fait
		// West->Sud : deja fait
		// North->Sud
		propagW[s][n] = gsl_matrix_complex_concat(
				cgemm_concat(btmA[s][w], MM[0][4], MM[0][5], btmA[s][n], NULL),
				cgemm_concat(btmB[s][e], MM[1][4], MM[1][5], NULL, btmB[s][n]),
				VERTI);
//				ConcatGmm(btmA[s][w], MM[0][4], MM[0][5], btmA[s][n], false),
//				ConcatGmm(btmB[s][e], MM[1][4], MM[1][5], btmB[s][n], true),
//				true);

		//		ScatMat[s][n] = new ComplexArray2D(
		//				ScatMatA[s][n].size(0) + ScatMatB[s][n].size(0),
		//				ScatMatA[s][n].size(0) + ScatMatB[s][n].size(0));
		//		ComplexArray2D matTmp1 = ScatMat[s][n].viewPart(0, 0,
		//				ScatMatA[s][n].size(0), ScatMat[s][n].size(1));
		//		ComplexArray2D matTmp2 = ScatMat[s][n].viewPart(ScatMatA[s][n].size(0),
		//				0, ScatMatB[s][n].size(0), ScatMat[s][n].size(1));
		//		matTmp1.AssignMultConcat(ScatMatA[s][w], MM[0][4], MM[0][5],
		//				ScatMatA[s][n], null, 1, 1, false, false);
		//		matTmp2.AssignMultConcat(ScatMatB[s][e], MM[1][4], MM[1][5], null,
		//				ScatMatB[s][n], 1, 1, false, false);

		// Sud->Sud
		if (bounds[n]) { // si il y a un voisin au nord
			propagW[s][s] = gsl_matrix_complex_concat(
					cgemm_concat(btmA[s][w], MM[0][2], MM[0][3], btmA[s][s],
							NULL),
					cgemm_concat(btmB[s][e], MM[1][2], MM[1][3], NULL,
							btmB[s][s]), VERTI);
//			propagW[s][s] = ConcatMatrix(
//					ConcatGmm(btmA[s][w], MM[0][2], MM[0][3], btmA[s][s],
//							false),
//					ConcatGmm(btmB[s][e], MM[1][2], MM[1][3], btmB[s][s], true),
//					true);

			//			ScatMat[s][s] = new ComplexArray2D(
			//					ScatMatA[s][s].size(0) + ScatMatB[s][s].size(0),
			//					ScatMatA[s][s].size(0) + ScatMatB[s][s].size(0));
			//			matTmp1 = ScatMat[s][s].viewPart(0, 0, ScatMatA[s][s].size(0),
			//					ScatMat[s][s].size(1));
			//			matTmp2 = ScatMat[s][s].viewPart(ScatMatA[s][s].size(0), 0,
			//					ScatMatB[s][s].size(0), ScatMat[s][s].size(1));
			//			matTmp1.AssignMultConcat(ScatMatA[s][w], MM[0][2], MM[0][3],
			//					ScatMatA[s][s], null, 1, 1, false, false);
			//			matTmp2.AssignMultConcat(ScatMatB[s][e], MM[1][2], MM[1][3], null,
			//					ScatMatB[s][s], 1, 1, false, false);
			propagW[n][n] = gsl_matrix_complex_alloc(propagW[s][s]->size2,
					propagW[s][s]->size1);
			gsl_matrix_complex_transpose_memcpy(propagW[n][n], propagW[s][s]);

			//ScatMat[n][n] = ScatMat[s][s].transpose();
		}
	}
	// propag vers le nord
	if (bounds[n]) { // si voisin Nord peut existergsl_matrix_complex_get(m, i, j).dat[0]
		// Est->Nord
		// West->Nord,
		// North->Nord
		// Sud->Nord
		propagW[n][s] = gsl_matrix_complex_concat(
				cgemm_concat(btmA[n][w], MM[0][2], MM[0][3], btmA[n][s], NULL),
				cgemm_concat(btmB[n][e], MM[1][2], MM[1][3], NULL, btmB[n][s]),
				VERTI);
//		propagW[n][s] = ConcatMatrix(
//				ConcatGmm(btmA[n][w], MM[0][2], MM[0][3], btmA[n][s], false),
//				ConcatGmm(btmB[n][e], MM[1][2], MM[1][3], btmB[n][s], true),
//				true);

		//		ScatMat[n][s] = new ComplexArray2D(
		//				ScatMatA[n][s].size(0) + ScatMatB[n][s].size(0),
		//				ScatMatA[n][s].size(0) + ScatMatB[n][s].size(0));
		//		ComplexArray2D matTmp1 = ScatMat[n][s].viewPart(0, 0,
		//				ScatMatA[n][s].size(0), ScatMat[n][s].size(1));
		//		ComplexArray2D matTmp2 = ScatMat[n][s].viewPart(ScatMatA[n][s].size(0),
		//				0, ScatMatB[n][s].size(0), ScatMat[n][s].size(1));
		//		matTmp1.AssignMultConcat(ScatMatA[n][w], MM[0][2], MM[0][3],
		//				ScatMatA[n][s], null, 1, 1, false, false);
		//		matTmp2.AssignMultConcat(ScatMatB[n][e], MM[1][2], MM[1][3], null,
		//				ScatMatB[n][s], 1, 1, false, false);
	}
	for (int i = 0; i < 4; i++) {
		std::vector<gsl_matrix_complex> mc_tmp;
		for (int j = 0; j < 4; j++) {
			bm.W[i][j] = propagW[i][j];
		}
	}
	bm.propagSource_AtoB = gsl_matrix_complex_alloc(MM[1][7]->size1,
			MM[1][7]->size2);
	gsl_matrix_complex_memcpy(bm.propagSource_AtoB, MM[1][7]);
	return bm;
}
gsl_matrix_complex *BlocType::gsl_matrix_complex_invert(gsl_matrix_complex *m) {
	gsl_matrix_complex *inverse = gsl_matrix_complex_alloc(m->size1, m->size2);
	gsl_permutation *perm = gsl_permutation_alloc(m->size1);
	int s = 0;
	gsl_linalg_complex_LU_decomp(m, perm, &s);
	gsl_linalg_complex_LU_invert(m, perm, inverse);
	//gsl_matrix_complex_fprintf(stdout, inverse, "%f");
	return inverse;
}
gsl_matrix_complex *BlocType::gsl_matrix_complex_concat(gsl_matrix_complex *A,
		gsl_matrix_complex *B, bool dir) {
	gsl_matrix_complex *C;
	gsl_matrix_complex_view aview, bview;
	if (dir == HORIZ) {
		C = gsl_matrix_complex_alloc(A->size1, A->size2 + B->size2);
		aview = gsl_matrix_complex_submatrix(C, 0, 0, A->size1, A->size2);
		bview = gsl_matrix_complex_submatrix(C, 0, A->size2, B->size1,
				B->size2);
	} else {
		C = gsl_matrix_complex_alloc(A->size1 + B->size1, A->size2);
		aview = gsl_matrix_complex_submatrix(C, 0, 0, A->size1, A->size2);
		bview = gsl_matrix_complex_submatrix(C, A->size1, 0, B->size1,
				B->size2);
	}
	gsl_matrix_complex_memcpy(&aview.matrix, A);
	gsl_matrix_complex_memcpy(&bview.matrix, B);
	return C;
}
gsl_matrix_complex *BlocType::cgemm_concat(gsl_matrix_complex *A,
		gsl_matrix_complex *B1, gsl_matrix_complex *B2, gsl_matrix_complex *C1,
		gsl_matrix_complex *C2) {
	gsl_matrix_complex *BB = gsl_matrix_complex_concat(B1, B2, HORIZ);
	gsl_matrix_complex *CC = gsl_matrix_complex_alloc(A->size1, BB->size2);
	gsl_matrix_complex_set_zero(CC);
	gsl_matrix_complex_view Cview;
	if (C1 == NULL) {
		Cview = gsl_matrix_complex_submatrix(CC, 0, CC->size2 - C2->size2,
				CC->size1, C2->size2);
		gsl_matrix_complex_memcpy(&Cview.matrix, C2);
	} else {
		Cview = gsl_matrix_complex_submatrix(CC, 0, 0, CC->size1, C1->size2);
		gsl_matrix_complex_memcpy(&Cview.matrix, C1);
	}
	cgemm(CblasNoTrans, CblasNoTrans, gsl_complex_rect(1, 0), A, BB,
			gsl_complex_rect(1, 0), CC);
	return CC;
}

void BlocType::setPropagDownMatrix(BlocType *childl, BlocType *childr) {
	bm.propagSource_BtoA = gsl_matrix_complex_alloc(bm.propagSource_AtoB->size2,
			bm.propagSource_AtoB->size1);
	gsl_matrix_complex_transpose_memcpy(bm.propagSource_BtoA,
			bm.propagSource_AtoB);
	if (this->splitDir == HORIZ) {
		//if (childl->bm.W != NULL) {
		bm.propagDown_ToB[S] = childl->bm.W[E][S];
		bm.propagDown_ToB[E] = childl->bm.W[E][E];
		bm.propagDown_ToB[N] = childl->bm.W[E][N];
		bm.propagSource_AtoA = childl->bm.W[E][W];
		//}
		//if (childr->bm.W != NULL) {
		bm.propagDown_ToA[S] = childr->bm.W[W][S];
		bm.propagDown_ToA[W] = childr->bm.W[W][W];
		bm.propagDown_ToA[N] = childr->bm.W[W][N];
		bm.propagSource_BtoB = childr->bm.W[W][E];
		//}

	} else {
		//if (childl->bm.W != NULL) {
		bm.propagDown_ToB[E] = childl->bm.W[S][E];
		bm.propagDown_ToB[W] = childl->bm.W[S][W];
		bm.propagDown_ToB[S] = childl->bm.W[S][S];
		bm.propagSource_AtoA = childl->bm.W[S][N];

		//}
		//if (childr->bm.W != NULL) {
		bm.propagDown_ToA[E] = childr->bm.W[N][E];
		bm.propagDown_ToA[W] = childr->bm.W[N][W];
		bm.propagDown_ToA[N] = childr->bm.W[N][N];
		bm.propagSource_BtoB = childr->bm.W[N][S];
		//}
	}
	downMatrix = true;
}

class BlocTypeDB {
	std::map<std::vector<size_t>, std::map<int, BlocType> >::iterator it_map;
public:
	size_t env_size1, env_size2;
	std::map<std::vector<size_t>, std::map<int, BlocType> > blocTypeDB;
	BlocTypeDB();
	virtual ~BlocTypeDB();
	void addMaterials(int num);
	void addBlocType(BlocType bt);
	BlocType* getBlocType(size_t size1, size_t size2, int indice);
	/*get bloctype with propagdown matrix*/
//	BlocType* getBlocType(size_t size1, size_t size2, int indice,
//			bool wihtDownMatrix);
};
BlocTypeDB::BlocTypeDB() {

}
BlocTypeDB::~BlocTypeDB() {
	// TODO Auto-generated destructor stub
}
void BlocTypeDB::addMaterials(int num) {
	for (int i = 0; i < num; i++) {
		BlocType mbt(1, 1, i);
		addBlocType(mbt);
	}
}
void BlocTypeDB::addBlocType(BlocType bt) {
	std::vector<size_t> key;
	key.push_back(bt.size1);
	key.push_back(bt.size2);
	if (blocTypeDB.count(key)) {
		it_map = blocTypeDB.find(key);
		it_map->second.insert(std::make_pair(bt.indice, bt));
	} else {
		std::map<int, BlocType> bt_map;
		bt_map.insert(std::make_pair(bt.indice, bt));
		blocTypeDB.insert(std::make_pair(key, bt_map));
	}
}
BlocType* BlocTypeDB::getBlocType(size_t size1, size_t size2, int indice) {
	std::vector<size_t> key;
	key.push_back(size1);
	key.push_back(size2);
	if (blocTypeDB.count(key)) {
		it_map = blocTypeDB.find(key);
		if (it_map->second.count(indice))
			return &it_map->second.find(indice)->second;
		else
			return NULL;
	} else
		return NULL;
}

class PreprocessEngine {
	std::map<std::vector<size_t>, std::vector<std::vector<int> > > positionDB;

public:
	std::vector<std::vector<double> > materials;
	BlocTypeDB btdb;
	gsl_matrix *level0;
	double f0, c0, dr;
	size_t env_size1, env_size2;
	int num_material;

	PreprocessEngine();
	virtual ~PreprocessEngine();

	void init(const char* dir);
	void startPreprocessEngine();

	void buildIrregularPyramid();
	void setHomogenety(int row, int column, BlocType bt);
	int findIndice(int row, int column, size_t size1, size_t size2);
	bool compareArea(int row1, int column1, int row2, int column2, size_t size1,
			size_t size2);
	void setBounds(int row, int column, BlocType *bt);
	void updateBounds(int row, int column, BlocType *bt);
	int bestDiscontinuities(int row, int column, BlocType *bt, int dir);
	int getSquareSize(int size);
	void splitBloc(int row, int column, BlocType *bt);
	void splitBlocHoriz(int row, int column, BlocType *bt);
	void splitBlocVerti(int row, int column, BlocType *bt);

	void setParameters(double f0, double c0, double dr);
	void fillUnitMatrix();
	void calculUnitMatrix(double n, double alpha, int num);
	void fillBlocMatrix();
	bool preocessBlocType(BlocType *bt);
	void printMatrix(gsl_matrix_complex *m[4][4]);
};
PreprocessEngine::PreprocessEngine() {
	// TODO Auto-generated constructor stub

}
PreprocessEngine::~PreprocessEngine() {
	// TODO Auto-generated destructor stub
}
void PreprocessEngine::init(const char* dir) {
	EnvironmentFile env_file(dir);
	this->level0 = env_file.getEnvironmentMatrix();
	this->env_size1 = env_file.getSize1();
	this->env_size2 = env_file.getSize2();
	btdb.env_size1 = env_size1;
	btdb.env_size2 = env_size2;
	this->num_material = env_file.getMaterialNum();
	this->materials = env_file.getMatTable();
	this->setParameters(f0_test, C0, DR);
}
void PreprocessEngine::startPreprocessEngine() {
	btdb.addMaterials(num_material);
	BlocType *bloc = new BlocType(env_size1, env_size2, 0);
	splitBloc(0, 0, bloc);
	fillUnitMatrix();
	fillBlocMatrix();
}
void PreprocessEngine::setHomogenety(int row, int column, BlocType bt) {

}
int PreprocessEngine::findIndice(int row, int column, size_t size1,
		size_t size2) {
	int ind = -1;
	std::vector<size_t> key_size;
	key_size.push_back(size1);
	key_size.push_back(size2);
	if (size1 == 1 && size2 == 1)
		return gsl_matrix_get(level0, row, column);
	else {
		//Key key(size1, size2);
		if (positionDB.count(key_size)) {
			bool found = false;
			std::map<std::vector<size_t>, std::vector<std::vector<int> > >::iterator it_DB =
					positionDB.find(key_size);
			while ((!found) && ind < (int) (it_DB->second.size() - 1)) {
				ind++;
				int pos[2] = { it_DB->second[ind][0], it_DB->second[ind][1] };
				found = compareArea(pos[0], pos[1], row, column, size1, size2);
			}
			if (!found) {
				ind = it_DB->second.size();
				std::vector<int> pos;
				pos.push_back(row);
				pos.push_back(column);
				it_DB->second.push_back(pos);
				positionDB.insert(std::make_pair(key_size, it_DB->second));
			}
			return ind;
		} else {
			std::vector<int> pos;
			pos.push_back(row);
			pos.push_back(column);
			std::vector<std::vector<int> > positionVector;
			positionVector.push_back(pos);
			positionDB.insert(std::make_pair(key_size, positionVector));
//			std::map<std::vector<size_t>, std::vector<std::vector<int> > >::iterator it_DB =
//					positionDB.find(key_size);
//			std::cout << it_DB->second[0][0] << " " << it_DB->second[0][1]
//					<< std::endl;
			return 0;
		}
	}
}
bool PreprocessEngine::compareArea(int row1, int column1, int row2, int column2,
		size_t size1, size_t size2) {
	gsl_matrix_view bt1 = gsl_matrix_submatrix(level0, row1, column1, size1,
			size2);
	gsl_matrix_view bt2 = gsl_matrix_submatrix(level0, row2, column2, size1,
			size2);
	return gsl_matrix_equal(&bt1.matrix, &bt2.matrix);
}
void PreprocessEngine::setBounds(int row, int column, BlocType *bt) {
	if (bt->bounds[E] || (column + bt->size2 != env_size2))
		bt->bounds[E] = true;
	if (bt->bounds[W] || (column != 0))
		bt->bounds[W] = true;
	if (bt->bounds[S] || (row + bt->size1 != env_size1))
		bt->bounds[S] = true;
	if (bt->bounds[N] || (row != 0))
		bt->bounds[N] = true;
}
void PreprocessEngine::updateBounds(int row, int column, BlocType *bt) {
	if (bt->isUnitary() == false) {
		setBounds(row, column, bt);
		if (bt->splitDir == HORIZ) {
			updateBounds(row, column,
					btdb.getBlocType(bt->childL->size1, bt->childL->size2,
							bt->childL->indice));
			updateBounds(row, column + bt->childL->size2,
					btdb.getBlocType(bt->childR->size1, bt->childR->size2,
							bt->childR->indice));
		} else {
			updateBounds(row, column,
					btdb.getBlocType(bt->childL->size1, bt->childL->size2,
							bt->childL->indice));
			updateBounds(row + bt->childL->size1, column,
					btdb.getBlocType(bt->childR->size1, bt->childR->size2,
							bt->childR->indice));
		}
	}
}
int PreprocessEngine::bestDiscontinuities(int row, int column, BlocType *bt,
		int dir) {
	std::vector<int> dis;
	dis.clear();
	if (dir == VERTI) {
		for (int i = 0; i < (bt->size1 - 1); i++) {
			int countV = 0;
			for (size_t j = 0; j < bt->size2; j++) {
				if (gsl_matrix_get(level0, row + i, column + j)
						!= gsl_matrix_get(level0, row + i + 1, column + j))
					countV++;
			}
			dis.push_back(countV);
		}
	} else {
		std::vector<int> horizDis(bt->size2 - 1);
		for (int i = 0; i < (bt->size2 - 1); i++) {
			int countH = 0;
			for (size_t j = 0; j < bt->size1; j++) {
				if (gsl_matrix_get(level0, row + j, column + i)
						!= gsl_matrix_get(level0, row + j, column + i + 1))
					countH++;
			}
			dis.push_back(countH);
		}
	}
	int blocCenter = (dis.size() + 1) / 2;
	int maxD = 0;
	int cpt = 0;
	int dist2Cent = 5000;
	int indicem = 0;
//	float indice = 1.0;
//	int toleranceP = ceil(blocCenter + indice * blocCenter);
//	int toleranceM = ceil(blocCenter - indice * blocCenter);

	for (size_t i = 0; i < dis.size(); i++)
		if (dis[i] > maxD)
			maxD = dis[i];

	int off[dis.size()];
	for (size_t i = 0; i < dis.size(); i++)
		if (dis[i] == maxD)
			off[cpt++] = i;

	for (int i = 0; i < cpt; i++)
		if (abs(off[i] - blocCenter) < dist2Cent) {
			dist2Cent = abs(off[i] - blocCenter);
			indicem = off[i];
		}
	dis.clear();
	if (maxD == 0) {
		if (dir == HORIZ)
			return getSquareSize(bt->size2) - 1;
		else
			return getSquareSize(bt->size1) - 1;
	} else
		return indicem;
}
int PreprocessEngine::getSquareSize(int size) {
	int power = 1;
	while (power < size)
		power *= 2;
	power /= 2;
	return power;
}
void PreprocessEngine::splitBloc(int row, int column, BlocType *bt) {
	if (bt->isUnitary() == false) {
		setBounds(row, column, bt);
		if (bt->size2 > bt->size1) {
			bt->splitDir = HORIZ;
			splitBlocHoriz(row, column, bt);
		} else {
			bt->splitDir = VERTI;
			splitBlocVerti(row, column, bt);
		}
	}
	btdb.addBlocType(*bt);
}
void PreprocessEngine::splitBlocVerti(int row, int column, BlocType *bt) {
	int cuttingPoint = bestDiscontinuities(row, column, bt, VERTI);
	int childLId = findIndice(row, column, cuttingPoint + 1, bt->size2);
	BlocType *oldBt = btdb.getBlocType(cuttingPoint + 1, bt->size2, childLId);
	if (oldBt == NULL) {
		bt->childL = new BlocType(cuttingPoint + 1, bt->size2, childLId);
		splitBloc(row, column, bt->childL);
	} else {
		bt->childL = oldBt;
		updateBounds(cuttingPoint + 1, bt->size2, oldBt);
	}

	int childRId = findIndice(row + cuttingPoint + 1, column,
			bt->size1 - cuttingPoint - 1, bt->size2);
	oldBt = btdb.getBlocType(bt->size1 - cuttingPoint - 1, bt->size2, childRId);
	if (oldBt == NULL) {
		bt->childR = new BlocType(bt->size1 - cuttingPoint - 1, bt->size2,
				childRId);
		splitBloc(row + cuttingPoint + 1, column, bt->childR);
	} else {
		bt->childR = oldBt;
		updateBounds(bt->size1 - cuttingPoint - 1, bt->size2, oldBt);
	}
}
void PreprocessEngine::splitBlocHoriz(int row, int column, BlocType *bt) {
	int cuttingPoint = bestDiscontinuities(row, column, bt, HORIZ);
	int childLId = findIndice(row, column, bt->size1, cuttingPoint + 1);
	BlocType *oldBt = btdb.getBlocType(bt->size1, cuttingPoint + 1, childLId);
	if (oldBt == NULL) {
		bt->childL = new BlocType(bt->size1, cuttingPoint + 1, childLId);
		splitBloc(row, column, bt->childL);
	} else {
		bt->childL = oldBt;
		updateBounds(bt->size1, cuttingPoint + 1, oldBt);
	}

	int childRId = findIndice(row, column + cuttingPoint + 1, bt->size1,
			bt->size2 - cuttingPoint - 1);
	oldBt = btdb.getBlocType(bt->size1, bt->size2 - cuttingPoint - 1, childRId);
	if (oldBt == NULL) {
		bt->childR = new BlocType(bt->size1, bt->size2 - cuttingPoint - 1,
				childRId);
		splitBloc(row, column + cuttingPoint + 1, bt->childR);
	} else {
		bt->childR = oldBt;
		updateBounds(bt->size1, bt->size2 - cuttingPoint - 1, oldBt);
	}
}
void PreprocessEngine::setParameters(double f0, double c0, double dr) {
	this->f0 = f0;
	this->c0 = c0;
	this->dr = dr;
}
void PreprocessEngine::fillUnitMatrix() {
	std::vector<size_t> key_size;
	key_size.push_back(1);
	key_size.push_back(1);
	for (int i = 0; i < num_material; i++) {
		double n = materials[i][1];
		double alpha = materials[i][2];
		calculUnitMatrix(n, alpha, i);
		btdb.blocTypeDB.find(key_size)->second.find(i)->second.matrix = true;
		btdb.blocTypeDB.find(key_size)->second.find(i)->second.bounds[0] = true;
		btdb.blocTypeDB.find(key_size)->second.find(i)->second.bounds[1] = true;
		btdb.blocTypeDB.find(key_size)->second.find(i)->second.bounds[2] = true;
		btdb.blocTypeDB.find(key_size)->second.find(i)->second.bounds[3] = true;
	}
}
void PreprocessEngine::calculUnitMatrix(double n, double alpha, int num) {
	std::vector<size_t> key_size;
	key_size.push_back(1);
	key_size.push_back(1);

	float n2 = n * n;
	float Alpha = 1 - 2 * n2;
	float Beta = 2 * n2 - 4;
	float Gamma = 4 * (n2 - 1);
	float Delta_re = 1 / (2 * n2) * cos(-2 * PI * f0 * dr / c0 / sqrt(2.0));
	float Delta_im = 1 / (2 * n2) * sin(-2 * PI * f0 * dr / c0 / sqrt(2.0));
	gsl_complex Delta = gsl_complex_rect(Delta_re, Delta_im);
	//std::complex<double> Delta(Delta_re, Delta_im);
	gsl_complex Sigma_ii = gsl_complex_mul_real(Delta, Beta);
	// k = Delta/(1-Beta*Delta)
	gsl_complex k = gsl_complex_div(Delta,
			gsl_complex_sub(gsl_complex_rect(1, 0), Sigma_ii));
	// Delta_1 = 1+Gamma*k
	gsl_complex Delta_1 = gsl_complex_add_real(gsl_complex_mul_real(k, Gamma),
			1);
	// Delta_2 = Alpha+Gamma*k
	gsl_complex Delta_2 = gsl_complex_add_real(gsl_complex_mul_real(k, Gamma),
			Alpha);
	gsl_matrix_complex *mat_tmp = gsl_matrix_complex_alloc(4, 4);
	gsl_matrix_complex_set_all(mat_tmp, Delta_1);
	gsl_matrix_complex_set(mat_tmp, 0, 1, Delta_2);
	gsl_matrix_complex_set(mat_tmp, 1, 0, Delta_2);
	gsl_matrix_complex_set(mat_tmp, 3, 2, Delta_2);
	gsl_matrix_complex_set(mat_tmp, 2, 3, Delta_2);
	gsl_matrix_complex_scale(mat_tmp, Delta);
	gsl_matrix_complex_scale(mat_tmp, gsl_complex_rect(alpha, 0));
	for (int i = 0; i < 4; i++) {
		for (int j = 0; j < 4; j++) {
			gsl_matrix_complex *tmp2 = gsl_matrix_complex_alloc(1, 1);
			gsl_matrix_complex_set(tmp2, 0, 0,
					gsl_matrix_complex_get(mat_tmp, i, j));
			btdb.blocTypeDB.find(key_size)->second.find(num)->second.bm.W[i][j] =
					gsl_matrix_complex_alloc(1, 1);
			gsl_matrix_complex_memcpy(
					btdb.blocTypeDB.find(key_size)->second.find(num)->second.bm.W[i][j],
					tmp2);
			gsl_matrix_complex_free(tmp2);
		}
	}
	//printMatrix(btdb.blocTypeDB.find(key_size)->second.find(num)->second.bm.W);
	btdb.blocTypeDB.find(key_size)->second.find(num)->second.matrix = true;
}
void PreprocessEngine::fillBlocMatrix() {
	std::vector<size_t> head_key;
	head_key.push_back(env_size1);
	head_key.push_back(env_size2);
	BlocType *headBloc = &btdb.blocTypeDB.find(head_key)->second.find(0)->second;
	preocessBlocType(headBloc);
}
bool PreprocessEngine::preocessBlocType(BlocType *bt) {
	BlocType *childL = btdb.getBlocType(bt->childL->size1, bt->childL->size2,
			bt->childL->indice);
	if (!childL->matrix) {
		if (preocessBlocType(childL) == false)
			return false;
	}
	BlocType *childR = btdb.getBlocType(bt->childR->size1, bt->childR->size2,
			bt->childR->indice);
	if (!childR->matrix) {
		if (preocessBlocType(childR) == false)
			return false;
	}
	//clock_t start = clock();
	bt->bm = bt->calculBlocMatrix(childL->bm.W, childR->bm.W);
	//clock_t finish = clock();

//	if (bt->size1 > 100 && bt->size2 > 100) {
//		std::cout << bt->size1 << " " << bt->size2 << " "
//				<< (finish - start) / 1000 << std::endl;
//	}

	//printMatrix(bt->bm.W);
	bt->matrix = true;
	return true;
}

class Bloc {
public:
	int row, column;
	size_t size1, size2;
	BlocType *bt;
	Bloc *childL, *childR, *parent;
	int indice;

	Bloc(int row, int column, size_t size1, size_t size2);
	virtual ~Bloc();
	bool isUnitary() {
		if (this->size1 == 1 && this->size2 == 1)
			return true;
		else
			return false;
	}
	bool isChildL() {
		if (this->parent->childL == this)
			return true;
		else
			return false;
	}
	void splitBloc();
};
Bloc::Bloc(int row, int column, size_t size1, size_t size2) {
	this->row = row;
	this->column = column;
	this->size1 = size1;
	this->size2 = size2;
	this->childL = NULL;
	this->childR = NULL;
	this->parent = NULL;
	bt = NULL;
}
Bloc::~Bloc() {
	// TODO Auto-generated destructor stub
}

class Flux {
public:
	gsl_matrix_complex *MatrixFlowE;
	gsl_matrix_complex *MatrixFlowW;
	gsl_matrix_complex *MatrixFlowS;
	gsl_matrix_complex *MatrixFlowN;

	Flux();
	virtual ~Flux();
	void init(size_t size1, size_t size2);
	gsl_vector_complex_view flowView(Bloc *bloc, int sens, int cardinal);
	gsl_vector_complex* Flow(Bloc* bloc, int sens, int cardinal);
};
Flux::Flux() {

}
Flux::~Flux() {
	// TODO Auto-generated destructor stub
}
void Flux::init(size_t size1, size_t size2) {
	MatrixFlowE = gsl_matrix_complex_alloc(size1 + 1, size2 + 1);
	//gsl_matrix_complex_transpose(MatrixFlowE);
	MatrixFlowW = gsl_matrix_complex_alloc(size1 + 1, size2 + 1);
	//gsl_matrix_complex_transpose(MatrixFlowW);
	MatrixFlowS = gsl_matrix_complex_alloc(size1 + 1, size2 + 1);
	MatrixFlowN = gsl_matrix_complex_alloc(size1 + 1, size2 + 1);

	gsl_matrix_complex_set_zero(MatrixFlowE);
	gsl_matrix_complex_set_zero(MatrixFlowW);
	gsl_matrix_complex_set_zero(MatrixFlowS);
	gsl_matrix_complex_set_zero(MatrixFlowN);
}
gsl_vector_complex_view Flux::flowView(Bloc *bloc, int sens, int cardinal) {
	switch (cardinal) {
	case E:
		if (sens == OUT)
			return gsl_matrix_complex_subcolumn(MatrixFlowE,
					bloc->column + bloc->size2, bloc->row, bloc->size1);
		else
			return gsl_matrix_complex_subcolumn(MatrixFlowE, bloc->column,
					bloc->row, bloc->size1);
	case W:
		if (sens == OUT)
			return gsl_matrix_complex_subcolumn(MatrixFlowW, bloc->column,
					bloc->row, bloc->size1);
		else
			return gsl_matrix_complex_subcolumn(MatrixFlowW,
					bloc->column + bloc->size2, bloc->row, bloc->size1);
	case S:
		if (sens == OUT)
			return gsl_matrix_complex_subrow(MatrixFlowS,
					bloc->row + bloc->size1, bloc->column, bloc->size2);
		else
			return gsl_matrix_complex_subrow(MatrixFlowS, bloc->row,
					bloc->column, bloc->size2);
	case N:
		if (sens == OUT)
			return gsl_matrix_complex_subrow(MatrixFlowN, bloc->row,
					bloc->column, bloc->size2);
		else
			return gsl_matrix_complex_subrow(MatrixFlowN,
					bloc->row + bloc->size1, bloc->column, bloc->size2);
	}
	return gsl_matrix_complex_subrow(MatrixFlowE, 0, 0, 0);
}

class Coverage {
	Flux f;
	size_t env_size1;
	size_t env_size2;
public:
	Coverage(size_t env_size1, size_t env_size2);
	virtual ~Coverage();

	gsl_matrix * getTreeLeaves(Flux environment);
	float getPsi(int row, int column);
};
Coverage::Coverage(size_t env_size1, size_t env_size2) {
	this->env_size1 = env_size1;
	this->env_size2 = env_size2;

}
Coverage::~Coverage() {
	// TODO Auto-generated destructor stub
}
gsl_matrix * Coverage::getTreeLeaves(Flux environment) {
	this->f = environment;
	gsl_matrix * psi;
	psi = gsl_matrix_alloc(env_size1, env_size2);
	//gsl_matrix_complex *psiC = gsl_matrix_complex_alloc(env_size1, env_size2);
	for (size_t i = 0; i < env_size1; i++)
		for (size_t j = 0; j < env_size2; j++) {
			gsl_matrix_set(psi, i, j, getPsi(i, j) - 50);
		}
	return psi;
}
float Coverage::getPsi(int row, int column) {
	double psi = 0;
	gsl_complex psiC = gsl_complex_rect(0, 0);
	Bloc tmp(row, column, 1, 1);
	gsl_vector_complex_view fv;
	fv = f.flowView(&tmp, IN, S);
	psiC = gsl_complex_add(gsl_vector_complex_get(&fv.vector, 0), psiC);
	fv = f.flowView(&tmp, IN, N);
	psiC = gsl_complex_add(gsl_vector_complex_get(&fv.vector, 0), psiC);
	fv = f.flowView(&tmp, IN, W);
	psiC = gsl_complex_add(gsl_vector_complex_get(&fv.vector, 0), psiC);
	fv = f.flowView(&tmp, IN, E);
	psiC = gsl_complex_add(gsl_vector_complex_get(&fv.vector, 0), psiC);
	psi = gsl_complex_abs(psiC) * gsl_complex_abs(psiC);
	psi = 10 * log(psi / 4) / log(10);
	return psi;
}

class Pyramid {
public:
	size_t env_size1, env_size2;
	Flux environment;
	Bloc *headBloc;
	BlocTypeDB btdb;
	gsl_vector_complex_view A;
	gsl_vector_complex_view B;
	gsl_vector_complex *Acopy;
	Pyramid();
	virtual ~Pyramid();
	void initPyramid(BlocTypeDB btdb);
	gsl_vector_complex* Flow(Bloc *bloc, int sens, int cardinal);
	void propagUpSource(Bloc *bloc, bool isChildL);
	void propagUpSourceHorizL(Bloc *bloc);
	void propagUpSourceHorizR(Bloc *bloc);
	void propagUpSourceVertiL(Bloc *bloc);
	void propagUpSourceVertiR(Bloc *bloc);
	void propagDownFlux(Bloc *hb);
	void createBinaryTree(Bloc *headBloc);

	void printMatrix(gsl_matrix_complex *matrix);
};
Pyramid::Pyramid() {

}
Pyramid::~Pyramid() {
	// TODO Auto-generated destructor stub
}
void Pyramid::initPyramid(BlocTypeDB btdb) {
	Acopy = NULL;
	this->btdb = btdb;
	this->env_size1 = btdb.env_size1;
	this->env_size2 = btdb.env_size2;
	environment.init(env_size1, env_size2);
	headBloc = new Bloc(0, 0, env_size1, env_size2);
	headBloc->bt = btdb.getBlocType(env_size1, env_size2, 0);
	createBinaryTree(headBloc);
}
void Pyramid::propagUpSource(Bloc *bloc, bool isChildL) {
//	printMatrix(environment.MatrixFlowE);
//	printMatrix(environment.MatrixFlowW);
//	printMatrix(environment.MatrixFlowS);
//	printMatrix(environment.MatrixFlowN);
	bloc->bt = btdb.getBlocType(bloc->size1, bloc->size2, bloc->indice);
	if (isChildL) {
		if (bloc->bt->splitDir == HORIZ)
			propagUpSourceHorizL(bloc);
		else
			propagUpSourceVertiL(bloc);
	} else {
		if (bloc->bt->splitDir == HORIZ)
			propagUpSourceHorizR(bloc);
		else
			propagUpSourceVertiR(bloc);
	}
	if (bloc->parent != NULL)
		propagUpSource(bloc->parent, bloc->isChildL());
}
void Pyramid::propagUpSourceHorizL(Bloc *bloc) {
	//internal flow from childL to childR
	A = environment.flowView(bloc->childL, OUT, E);
	B = environment.flowView(bloc->childR, IN, E);
	Acopy = gsl_vector_complex_alloc(A.vector.size);
	gsl_vector_complex_memcpy(Acopy, &A.vector);
	cgemv(CblasNoTrans, gsl_complex_rect(1, 0), bloc->bt->bm.propagSource_AtoB,
			Acopy, gsl_complex_rect(0, 0), &B.vector);
	gsl_vector_complex_free(Acopy);

	// internal flow from childR to childL
	A = environment.flowView(bloc->childR, IN, E);
	B = environment.flowView(bloc->childR, OUT, W);
	Acopy = gsl_vector_complex_alloc(A.vector.size);
	gsl_vector_complex_memcpy(Acopy, &A.vector);
	cgemv(CblasNoTrans, gsl_complex_rect(1, 0), bloc->bt->bm.propagSource_BtoB,
			Acopy, gsl_complex_rect(0, 0), &B.vector);
	gsl_vector_complex_free(Acopy);

	if (bloc->column > 0) {
		// out flow of west
		A = environment.flowView(bloc->childL, IN, W);
		B = environment.flowView(bloc->childL, OUT, W);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[E], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}
	if (bloc->row + bloc->size1 < env_size1) {
		A = environment.flowView(bloc->childL, IN, W);
		B = environment.flowView(bloc->childL, OUT, S);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[N], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);

		A = environment.flowView(bloc->childR, IN, E);
		B = environment.flowView(bloc->childR, OUT, S);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[N], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}
	if (bloc->row > 0) {
		A = environment.flowView(bloc->childL, IN, W);
		B = environment.flowView(bloc->childL, OUT, N);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[S], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);

		A = environment.flowView(bloc->childR, IN, E);
		B = environment.flowView(bloc->childR, OUT, N);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[S], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}
	if (bloc->column + bloc->size2 < env_size2) {
		A = environment.flowView(bloc->childR, IN, E);
		B = environment.flowView(bloc->childR, OUT, E);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[W], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}
}
void Pyramid::propagUpSourceHorizR(Bloc *bloc) {
	A = environment.flowView(bloc->childR, OUT, W);
	B = environment.flowView(bloc->childL, IN, W);
	Acopy = gsl_vector_complex_alloc(A.vector.size);
	gsl_vector_complex_memcpy(Acopy, &A.vector);
	cgemv(CblasNoTrans, gsl_complex_rect(1, 0), bloc->bt->bm.propagSource_BtoA,
			Acopy, gsl_complex_rect(0, 0), &B.vector);
	gsl_vector_complex_free(Acopy);

	A = environment.flowView(bloc->childL, IN, W);
	B = environment.flowView(bloc->childL, OUT, E);
	Acopy = gsl_vector_complex_alloc(A.vector.size);
	gsl_vector_complex_memcpy(Acopy, &A.vector);
	cgemv(CblasNoTrans, gsl_complex_rect(1, 0), bloc->bt->bm.propagSource_AtoA,
			Acopy, gsl_complex_rect(0, 0), &B.vector);
	gsl_vector_complex_free(Acopy);

	if (bloc->column + bloc->size2 < env_size2) {
		A = environment.flowView(bloc->childR, IN, E);
		B = environment.flowView(bloc->childR, OUT, E);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[W], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}

	if (bloc->row + bloc->size1 < env_size1) {
		A = environment.flowView(bloc->childL, IN, W);
		B = environment.flowView(bloc->childL, OUT, S);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[N], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);

		A = environment.flowView(bloc->childR, IN, E);
		B = environment.flowView(bloc->childR, OUT, S);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[N], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}
	if (bloc->row > 0) {
		A = environment.flowView(bloc->childL, IN, W);
		B = environment.flowView(bloc->childL, OUT, N);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[S], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);

		A = environment.flowView(bloc->childR, IN, E);
		B = environment.flowView(bloc->childR, OUT, N);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[S], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}
	if (bloc->column > 0) {
		A = environment.flowView(bloc->childL, IN, W);
		B = environment.flowView(bloc->childL, OUT, W);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[E], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}
}
void Pyramid::propagUpSourceVertiL(Bloc *bloc) {
	// internal flow from childL to childR
	A = environment.flowView(bloc->childL, OUT, S);
	B = environment.flowView(bloc->childR, IN, S);
	Acopy = gsl_vector_complex_alloc(A.vector.size);
	gsl_vector_complex_memcpy(Acopy, &A.vector);
	cgemv(CblasNoTrans, gsl_complex_rect(1, 0), bloc->bt->bm.propagSource_AtoB,
			Acopy, gsl_complex_rect(0, 0), &B.vector);
	gsl_vector_complex_free(Acopy);

	//gsl_vector_complex_fprintf(stdout, &B.vector, "%f");

	// internal flow from childR to childL
	A = environment.flowView(bloc->childR, IN, S);
	B = environment.flowView(bloc->childR, OUT, N);
	Acopy = gsl_vector_complex_alloc(A.vector.size);
	gsl_vector_complex_memcpy(Acopy, &A.vector);
	cgemv(CblasNoTrans, gsl_complex_rect(1, 0), bloc->bt->bm.propagSource_BtoB,
			Acopy, gsl_complex_rect(0, 0), &B.vector);
	gsl_vector_complex_free(Acopy);
	//gsl_vector_complex_fprintf(stdout, &B.vector, "%f");

	if (bloc->row > 0) {
		// out flow of north
		A = environment.flowView(bloc->childL, IN, N);
		B = environment.flowView(bloc->childL, OUT, N);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[S], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}

	if (bloc->column + bloc->size2 < env_size2) {
		A = environment.flowView(bloc->childL, IN, N);
		B = environment.flowView(bloc->childL, OUT, E);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[W], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);

		A = environment.flowView(bloc->childR, IN, S);
		B = environment.flowView(bloc->childR, OUT, E);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[W], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}

	if (bloc->column > 0) {
		A = environment.flowView(bloc->childL, IN, N);
		B = environment.flowView(bloc->childL, OUT, W);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[E], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);

		A = environment.flowView(bloc->childR, IN, S);
		B = environment.flowView(bloc->childR, OUT, W);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[E], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}

	if (bloc->row + bloc->size1 < env_size1) {
		A = environment.flowView(bloc->childR, IN, S);
		B = environment.flowView(bloc->childR, OUT, S);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[N], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}
}
void Pyramid::propagUpSourceVertiR(Bloc *bloc) {
	A = environment.flowView(bloc->childR, OUT, N);
	B = environment.flowView(bloc->childL, IN, N);
	Acopy = gsl_vector_complex_alloc(A.vector.size);
	gsl_vector_complex_memcpy(Acopy, &A.vector);
	cgemv(CblasNoTrans, gsl_complex_rect(1, 0), bloc->bt->bm.propagSource_BtoA,
			Acopy, gsl_complex_rect(0, 0), &B.vector);
	gsl_vector_complex_free(Acopy);

	A = environment.flowView(bloc->childL, IN, N);
	B = environment.flowView(bloc->childL, OUT, S);
	Acopy = gsl_vector_complex_alloc(A.vector.size);
	gsl_vector_complex_memcpy(Acopy, &A.vector);
	cgemv(CblasNoTrans, gsl_complex_rect(1, 0), bloc->bt->bm.propagSource_AtoA,
			Acopy, gsl_complex_rect(0, 0), &B.vector);
	gsl_vector_complex_free(Acopy);

	if (bloc->row + bloc->size1 < env_size1) {
		A = environment.flowView(bloc->childR, IN, S);
		B = environment.flowView(bloc->childR, OUT, S);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[N], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}

	if (bloc->column + bloc->size2 < env_size2) {
		A = environment.flowView(bloc->childL, IN, N);
		B = environment.flowView(bloc->childL, OUT, E);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[W], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);

		A = environment.flowView(bloc->childR, IN, S);
		B = environment.flowView(bloc->childR, OUT, E);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[W], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}

	if (bloc->column > 0) {
		A = environment.flowView(bloc->childL, IN, N);
		B = environment.flowView(bloc->childL, OUT, W);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[E], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);

		A = environment.flowView(bloc->childR, IN, S);
		B = environment.flowView(bloc->childR, OUT, W);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToA[E], Acopy, gsl_complex_rect(1, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}

	if (bloc->row > 0) {
		A = environment.flowView(bloc->childL, IN, N);
		B = environment.flowView(bloc->childL, OUT, N);
		Acopy = gsl_vector_complex_alloc(A.vector.size);
		gsl_vector_complex_memcpy(Acopy, &A.vector);
		cgemv(CblasTrans, gsl_complex_rect(1, 0),
				bloc->bt->bm.propagDown_ToB[S], Acopy, gsl_complex_rect(0, 0),
				&B.vector);
		gsl_vector_complex_free(Acopy);
	}
}
void Pyramid::propagDownFlux(Bloc *bloc) {
	if (bloc->isUnitary() == false) {
		bloc->bt = btdb.getBlocType(bloc->size1, bloc->size2, bloc->indice);
		if (bloc->bt->splitDir == HORIZ) {
			gsl_vector_complex *f_B = gsl_vector_complex_alloc(
					environment.flowView(bloc->childL, OUT, E).vector.size);
			gsl_vector_complex_set_zero(f_B);
			gsl_vector_complex *f_A = gsl_vector_complex_alloc(
					environment.flowView(bloc->childR, OUT, W).vector.size);
			gsl_vector_complex_set_zero(f_A);

			if (bloc->bt->bounds[W]) {
				A = environment.flowView(bloc->childL, IN, E);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToB[E], Acopy,
						gsl_complex_rect(0, 0), f_B);
			}

			if (bloc->bt->bounds[E]) {
				A = environment.flowView(bloc->childR, IN, W);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToA[W], Acopy,
						gsl_complex_rect(0, 0), f_A);
			}

			if (bloc->bt->bounds[N]) {
				A = environment.flowView(bloc->childL, IN, S);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToB[S], Acopy,
						gsl_complex_rect(1, 0), f_B);

				A = environment.flowView(bloc->childR, IN, S);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToA[S], Acopy,
						gsl_complex_rect(1, 0), f_A);
			}

			if (bloc->bt->bounds[S]) {
				A = environment.flowView(bloc->childL, IN, N);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToB[N], Acopy,
						gsl_complex_rect(1, 0), f_B);

				A = environment.flowView(bloc->childR, IN, N);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToA[N], Acopy,
						gsl_complex_rect(1, 0), f_A);
			}

			Acopy = gsl_vector_complex_alloc(f_B->size);
			gsl_vector_complex_memcpy(Acopy, f_B);
			cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
					bloc->bt->bm.propagSource_AtoB, Acopy,
					gsl_complex_rect(0, 0), f_B);

			Acopy = gsl_vector_complex_alloc(f_A->size);
			gsl_vector_complex_memcpy(Acopy, f_A);
			cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
					bloc->bt->bm.propagSource_BtoA, Acopy,
					gsl_complex_rect(0, 0), f_A);

			A = environment.flowView(bloc->childL, IN, W);
			gsl_vector_complex_add(&A.vector, f_A);
			cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
					bloc->bt->bm.propagSource_BtoB, f_B, gsl_complex_rect(1, 0),
					&A.vector);

			A = environment.flowView(bloc->childR, IN, E);
			gsl_vector_complex_add(&A.vector, f_B);
			cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
					bloc->bt->bm.propagSource_AtoA, f_A, gsl_complex_rect(1, 0),
					&A.vector);
		} else {
			gsl_vector_complex *f_B = gsl_vector_complex_alloc(
					environment.flowView(bloc->childR, IN, S).vector.size);
			gsl_vector_complex_set_zero(f_B);
			gsl_vector_complex *f_A = gsl_vector_complex_alloc(
					environment.flowView(bloc->childL, IN, N).vector.size);
			gsl_vector_complex_set_zero(f_A);

			if (bloc->bt->bounds[N]) {
				A = environment.flowView(bloc->childL, IN, S);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToB[S], Acopy,
						gsl_complex_rect(0, 0), f_B);
			}

			if (bloc->bt->bounds[S]) {
				A = environment.flowView(bloc->childR, IN, N);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToA[N], Acopy,
						gsl_complex_rect(0, 0), f_A);
			}

			if (bloc->bt->bounds[W]) {
				A = environment.flowView(bloc->childL, IN, E);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToB[E], Acopy,
						gsl_complex_rect(1, 0), f_B);

				A = environment.flowView(bloc->childR, IN, E);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToA[E], Acopy,
						gsl_complex_rect(1, 0), f_A);
			}

			if (bloc->bt->bounds[E]) {
				A = environment.flowView(bloc->childL, IN, W);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToB[W], Acopy,
						gsl_complex_rect(1, 0), f_B);

				A = environment.flowView(bloc->childR, IN, W);
				Acopy = gsl_vector_complex_alloc(A.vector.size);
				gsl_vector_complex_memcpy(Acopy, &A.vector);
				cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
						bloc->bt->bm.propagDown_ToA[W], Acopy,
						gsl_complex_rect(1, 0), f_A);
			}

			Acopy = gsl_vector_complex_alloc(f_B->size);
			gsl_vector_complex_memcpy(Acopy, f_B);
			cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
					bloc->bt->bm.propagSource_AtoB, Acopy,
					gsl_complex_rect(0, 0), f_B);

			Acopy = gsl_vector_complex_alloc(f_A->size);
			gsl_vector_complex_memcpy(Acopy, f_A);
			cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
					bloc->bt->bm.propagSource_BtoA, Acopy,
					gsl_complex_rect(0, 0), f_A);

			A = environment.flowView(bloc->childL, IN, N);
			gsl_vector_complex_add(&A.vector, f_A);
			cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
					bloc->bt->bm.propagSource_BtoB, f_B, gsl_complex_rect(1, 0),
					&A.vector);

			A = environment.flowView(bloc->childR, IN, S);
			gsl_vector_complex_add(&A.vector, f_B);
			cgemv(CblasNoTrans, gsl_complex_rect(1, 0),
					bloc->bt->bm.propagSource_AtoA, f_A, gsl_complex_rect(1, 0),
					&A.vector);
		}
		propagDownFlux(bloc->childL);
		propagDownFlux(bloc->childR);
	}
}
void Pyramid::createBinaryTree(Bloc* bloc) {
	if (bloc->bt->isUnitary() == false) {
		bloc->childL = new Bloc(bloc->row, bloc->column,
				bloc->bt->childL->size1, bloc->bt->childL->size2);
		bloc->childL->bt = btdb.getBlocType(bloc->childL->size1,
				bloc->childL->size2, bloc->bt->childL->indice);
		bloc->childL->indice = bloc->bt->childL->indice;
		bloc->childL->parent = bloc;
		createBinaryTree(bloc->childL);

		if (bloc->bt->splitDir == HORIZ) {
			bloc->childR = new Bloc(bloc->row,
					bloc->column + bloc->childL->size2, bloc->bt->childR->size1,
					bloc->bt->childR->size2);
		} else {
			bloc->childR = new Bloc(bloc->row + bloc->bt->childL->size1,
					bloc->column, bloc->bt->childR->size1,
					bloc->bt->childR->size2);
		}
		bloc->childR->bt = btdb.getBlocType(bloc->childR->size1,
				bloc->childR->size2, bloc->bt->childR->indice);
		bloc->childR->indice = bloc->bt->childR->indice;
		createBinaryTree(bloc->childR);
		bloc->childR->parent = bloc;
	}
}
void Pyramid::printMatrix(gsl_matrix_complex *matrix) {
	std::cout << "[" << matrix->size1 << "," << matrix->size2 << "]"
			<< std::endl;
	for (size_t i = 0; i < matrix->size1; i++) {
		for (size_t j = 0; j < matrix->size2; j++) {
			std::cout << "(" << gsl_matrix_complex_get(matrix, i, j).dat[0]
					<< ", " << gsl_matrix_complex_get(matrix, i, j).dat[1]
					<< ")";
		}
		std::cout << std::endl;
	}
	std::cout << std::endl;
}

class PropagationEngine {
	BlocTypeDB btdb;
public:
	size_t env_size1, env_size2;
	Bloc *sourceBloc;
	Pyramid pyramid;
	Bloc *headBloc;
	PropagationEngine();
	virtual ~PropagationEngine();
	void init(BlocTypeDB btdb, Source s);
	gsl_matrix * startPropagEngine();
	void writeSourceInEnvironment(Source s);
	Bloc *getSourceBloc(Source s);
	void setPropagDownMatrix();
};
PropagationEngine::PropagationEngine() {
	// TODO Auto-generated constructor stub

}
PropagationEngine::~PropagationEngine() {
	// TODO Auto-generated destructor stub
}
void PropagationEngine::init(BlocTypeDB btdb, Source s) {
	this->btdb = btdb;
	std::map<std::vector<size_t>, std::map<int, BlocType> >::iterator it_map =
			btdb.blocTypeDB.begin();
	std::map<int, BlocType>::iterator it_bt;
	while (it_map != btdb.blocTypeDB.end()) {
		//std::cout << it_map->first << " " << it_bt->first << std::endl;
		it_bt = it_map->second.begin();
		while (it_bt != it_map->second.end()) {
			if (it_bt->second.isUnitary() == false) {
//				std::cout << it_bt->second.size1 << " " << it_bt->second.size2
//						<< " " << it_bt->second.indice << std::endl;
				BlocType *childL = btdb.getBlocType(it_bt->second.childL->size1,
						it_bt->second.childL->size2,
						it_bt->second.childL->indice);
				BlocType *childR = btdb.getBlocType(it_bt->second.childR->size1,
						it_bt->second.childR->size2,
						it_bt->second.childR->indice);
				it_bt->second.setPropagDownMatrix(childL, childR);
			}
			++it_bt;
		}
		++it_map;
	}
	pyramid.initPyramid(btdb);
	headBloc = pyramid.headBloc;
	env_size1 = btdb.env_size1;
	env_size2 = btdb.env_size2;
	/*	Source s(13, 25, 1, 1);
	 s.amplitude = 9.440609281128907;
	 s.phase = 1.0;*/
	writeSourceInEnvironment(s);
	sourceBloc = getSourceBloc(s);
}
gsl_matrix * PropagationEngine::startPropagEngine() {
	pyramid.propagUpSource(sourceBloc->parent, sourceBloc->isChildL());
	/*	pyramid.printMatrix(pyramid.environment.MatrixFlowE);
	 pyramid.printMatrix(pyramid.environment.MatrixFlowW);
	 pyramid.printMatrix(pyramid.environment.MatrixFlowS);
	 pyramid.printMatrix(pyramid.environment.MatrixFlowN);*/
	pyramid.propagDownFlux(headBloc);
	/*	pyramid.printMatrix(pyramid.environment.MatrixFlowE);
	 pyramid.printMatrix(pyramid.environment.MatrixFlowW);
	 pyramid.printMatrix(pyramid.environment.MatrixFlowS);
	 pyramid.printMatrix(pyramid.environment.MatrixFlowN);*/

	Coverage cov(env_size1, env_size2);
	gsl_matrix *cover = cov.getTreeLeaves(pyramid.environment);
//	for (size_t i = 0; i < env_size1; i++) {
//		for (size_t j = 0; j < env_size2; j++) {
//			std::cout << gsl_matrix_get(cover, i, j) << " ";
//		}
//		std::cout << std::endl;
//	}

	return cover;
}
void PropagationEngine::writeSourceInEnvironment(Source s) {
	double real = s.amplitude * cos(s.phase);
	double imag = s.amplitude * sin(s.phase);
	gsl_complex source = gsl_complex_rect(real, imag);
	gsl_matrix_complex_set(pyramid.environment.MatrixFlowE, s.row,
			s.column + s.size2, source);
	gsl_matrix_complex_set(pyramid.environment.MatrixFlowW, s.row, s.column,
			source);
	gsl_matrix_complex_set(pyramid.environment.MatrixFlowS, s.row + s.size1,
			s.column, source);
	gsl_matrix_complex_set(pyramid.environment.MatrixFlowN, s.row, s.column,
			source);
	//pyramid.printMatrix(pyramid.environment.MatrixFlowE);
}
Bloc* PropagationEngine::getSourceBloc(Source s) {
	Bloc *unitBloc = headBloc;
	while (unitBloc->size1 != 1 || unitBloc->size2 != 1) {
		if (unitBloc->bt->splitDir == HORIZ) {
			if (s.column < (unitBloc->column + unitBloc->childL->size2))
				unitBloc = unitBloc->childL;
			else
				unitBloc = unitBloc->childR;
		} else {
			if (s.row < (unitBloc->row + unitBloc->childL->size1))
				unitBloc = unitBloc->childL;
			else
				unitBloc = unitBloc->childR;
		}
	}
	return unitBloc;
}

class MRFDPF {

public:
	gsl_matrix *coverage;
	BlocTypeDB btdb;
	MRFDPF(const char* dir, Source s);
	virtual ~MRFDPF();

	void calCoverage(Source s);
	double getReceivePower(int row, int column);
};
inline MRFDPF::MRFDPF(const char* dir, Source s) {
	PreprocessEngine ppe;
	ppe.init(dir);
//	clock_t start = clock();
	ppe.startPreprocessEngine();
//	clock_t finish = clock();
//	std::cout << (finish - start) / 1000 << std::endl;

	btdb = ppe.btdb;

//	std::cout << (finish - start) / 1000 << std::endl;
	calCoverage(s);
}
MRFDPF::~MRFDPF() {
}
void MRFDPF::calCoverage(Source s) {
	//BlocType *childL = btdb.getBlocType(1, 1, 0);
	PropagationEngine pge;
	pge.init(this->btdb, s);
	coverage = pge.startPropagEngine();
}
double MRFDPF::getReceivePower(int row, int column) {
	double rss = 0;
	rss = gsl_matrix_get(coverage, row, column);
	return rss;
}
namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED(wiplan);

wiplan::wiplan(){
}

wiplan::~wiplan(){
}

TypeId wiplan::GetTypeId(void) {
	static TypeId tid = TypeId("ns3::wiplan")

	.SetParent<PropagationLossModel>()

        .AddConstructor<wiplan>()

	.AddAttribute("Frequency", "The propagation frequency in Hz",
			DoubleValue(2450e6),
			MakeDoubleAccessor(&wiplan::m_frequency),
			MakeDoubleChecker<double>())

	/*	You have to define a Indoor environment type in the propagation-environment.h.
	 So in the method GetLoss(), it will search the environment file in the directory*/
	.AddAttribute("Environment", "Environment Scenario",
			EnumValue(IndoorEnvironment),
			MakeEnumAccessor(&wiplan::m_environment),
			MakeEnumChecker(UrbanEnvironment, "Urban", SubUrbanEnvironment,
					"SubUrban", OpenAreasEnvironment, "OpenAreas",
					IndoorEnvironment, "Indoor"))

					/*	.AddAttribute("CitySize", "Dimension of the city", EnumValue(LargeCity),
					 MakeEnumAccessor(&wiplan::m_citySize),
					 MakeEnumChecker(SmallCity, "Small", MediumCity, "Medium", LargeCity,
					 "Large"))*/;

	return tid;
}

double wiplan::GetLoss(Ptr<MobilityModel> a,
		Ptr<MobilityModel> b) const {
	double loss = 0.0;
	//double fmhz = m_frequency / 1e6;
	if (m_environment == IndoorEnvironment) {
		Source s(a->GetPosition().x, b->GetPosition().y, 1, 1);
		s.amplitude = 9.440609281128907;
		s.phase = 1.0;
		MRFDPF mf("src/wiplan/examples/stanford.coeff", s);
		loss = mf.getReceivePower((int) b->GetPosition().x,
				(int) b->GetPosition().y);
                std::cout << loss << std::endl;
		//the environment file stored in the dir
	}
	//call the wiplan model to calculate the pathloss by entering the postion of a and b
	//loss = wiplan(environment_file, m_frequency, a->GetPosition(), b->GetPosition(), dbm);
	return loss;
}

double wiplan::DoCalcRxPower(double txPowerDbm,
		Ptr<MobilityModel> a, Ptr<MobilityModel> b) const {
	return (GetLoss(a, b));
}

int64_t wiplan::DoAssignStreams(int64_t stream) {
	return 0;
}

void wiplan::GetCoverage(){
}

} /* namespace ns3 */


