/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
#ifndef __WIPLAN_H__
#define __WIPLAN_H__

#include "ns3/propagation-loss-model.h"
#include "ns3/propagation-environment.h"

namespace ns3 {

/**
 * \ingroup propagation
 *
 * \brief this class implements the MR-FDPF propagation loss model
 *
 * this class implements the MR-FDPF propagation loss model,
 * which is used to model Indoor environment
 */

class wiplan: public PropagationLossModel {
public:

        wiplan();
        virtual ~wiplan();
	// inherited from Object
	static TypeId GetTypeId(void);

	/**
	 * \param a the first mobility model
	 * \param b the second mobility model
	 *
	 * \return the loss in dBm for the propagation between
	 * the two given mobility models
	 */
	double GetLoss(Ptr<MobilityModel> a, Ptr<MobilityModel> b) const;

private:

	// inherited from PropagationLossModel
	virtual double DoCalcRxPower(double txPowerDbm, Ptr<MobilityModel> a,
			Ptr<MobilityModel> b) const;
	virtual int64_t DoAssignStreams(int64_t stream);

        void GetCoverage();

	EnvironmentType m_environment;
	double m_frequency; // frequency in MHz

};

}

#endif /* __WIPLAN_H__ */

